/*******************************************************************************
 *
 *  ACESS_RFID_PIC16F628A
 *
 *  Controle de acesso via tags (carto ou chaveiro RFID)
 *
 *  C/ Pic 16F628A  e mdulo RFID-RC522
 *
 *
 *  Objetivo: Usar tags RFID Mifare para acessar ambiente usando uma fechadura
 *  eltrica.
 *  Se uma tag tiver sido cadastrada, ao passar no leitor ir liberar acesso
 *  (acender tambm o led verde para indicar isto).
 *  Caso ainda no tenha sido cadastrada, sinalizar acendendo o led vermelho,
 *  sem liberar entrada.
 *
 *  Programar tag mestre: A primeira tag lida ser gravada como  a tag mestre.
 *  Com ela ser possvel cadastrar ou deletar outras tags.
 *
 *  Programar outras tags: Passe novamente  a tag mestre no leitor rc522.
 *  Comear a piscar o led azul. Passe uma tag no leitor. Se estiver cadastrada
 *  ir deleta-la. Se no for cadastrada  ir grava-la na EEprom.
 *  Aps gravar as outras tags, passe novamente a tag mestre para entrar em modo
 *  normal.
 *  O led azul ir parar de piscar e se manter aceso.

 *  Deletar todas as tags da EEprom do PIC: Aperte o boto 'reset' e 'deletar'
 *  e mantenha apertados. Solte apenas o boto 'reset' e mantenha apertado o
 *  boto 'deletar'(o led vermelho acender). Quando o led vermelho comear a
 *  piscar, solte o boto 'deletar'. Neste momento foi concluido o apagamento
 *  de todas as tags na EEprom (incluindo a tag mestre).

 *  Substituir a tag mestre por uma outra: Em modo normal de operao (led azul
 *  aceso), aperte o boto 'deletar' e mantenha apertado (led vermelho acende).
 *  Aguarde at o led vermelho comear a piscar. Solte o boto 'deletar'. Aps
 *  isto, o led azul comea a piscar indicando que a prxima tag a ser passada
 *  pelo leitor ser gravada como a tag mestre.
 *
 *  Autor: Claudio Lrios
 *  Data: 31/03/2020
 *  Blog Picsource
 *  OBS. ESTA ARQUIVO  DE NATUREZA DIDTICA
 *   SENDO PARTE INTEGRANTE DO BLOG 'PICSOURCE'


*******************************************************************************/


#include <16F628A.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPUT, NOPROTECT, NOBROWNOUT, MCLR, NOLVP, NOCPD

//BYTE
#byte portb = 0x06
#byte porta = 0x05
#byte trisa = 0x85
#byte trisb = 0x86
//BIT

#bit  sw_del  = 0x5.4  //interruptor de 'delete'
#bit  rele    = 0x5.0  //sada para ligar rele da fechadura
#bit  led_vm   = 0x5.1 //led vermelho
#bit  led_vd   = 0x5.2 //led verde
#bit  led_az   = 0x5.3 //led azul


#bit  rst     = 0x6.4 //reset do MFRC-522
#bit  sda     = 0x6.0 //SS (chip select) do MFRC-522
#bit  mosi    = 0x6.2 //MOSI (master out/ slave in)
#bit  miso    = 0x6.3 //MISO (master in/ slave out)
#bit  sck     = 0x6.1 //SCK (clock)

#define LED_LIGADO 0   //valor para acionamento do led
#define LED_DESLIGADO 1
#define RELE_LIGADO  1 //valor para acionamento do rele
#define RELE_DESLIGADO 0


//------------------MFRC522 Register---------------
//Page 0:Command and Status
#define     RESERVED00            0x00
#define     COMMANDREG            0x01
#define     COMMIENREG            0x02
#define     DIVLENREG             0x03
#define     COMMIRQREG            0x04
#define     DIVIRQREG             0x05
#define     ERRORREG              0x06
#define     STATUS1REG            0x07
#define     STATUS2REG            0x08
#define     FIFODATAREG           0x09
#define     FIFOLEVELREG          0x0A
#define     WATERLEVELREG         0x0B
#define     CONTROLREG            0x0C
#define     BITFRAMINGREG         0x0D
#define     COLLREG               0x0E
#define     RESERVED01            0x0F
//PAGE 1:Command
#define     RESERVED10            0x10
#define     MODEREG               0x11
#define     TXMODEREG             0x12
#define     RXMODEREG             0x13
#define     TXCONTROLREG          0x14
#define     TXAUTOREG             0x15
#define     TXSELREG              0x16
#define     RXSELREG              0x17
#define     RXTHRESHOLDREG        0x18
#define     DEMODREG              0x19
#define     RESERVED11            0x1A
#define     RESERVED12            0x1B
#define     MIFAREREG             0x1C
#define     RESERVED13            0x1D
#define     RESERVED14            0x1E
#define     SERIALSPEEDREG        0x1F
//PAGE 2:CFG
#define     RESERVED20            0x20
#define     CRCRESULTREGM         0x21
#define     CRCRESULTREGL         0x22
#define     RESERVED21            0x23
#define     MODWIDTHREG           0x24
#define     RESERVED22            0x25
#define     RFCFGREG              0x26
#define     GSNREG                0x27
#define     CWGSPREG              0x28
#define     MODGSPREG             0x29
#define     TMODEREG              0x2A
#define     TPRESCALERREG         0x2B
#define     TRELOADREGH           0x2C
#define     TRELOADREGL           0x2D
#define     TCOUNTERVALUEREGH     0x2E
#define     TCOUNTERVALUEREGL     0x2F
//PAGE 3:TEST REGISTER
#define     RESERVED30            0x30
#define     TESTSEL1REG           0x31
#define     TESTSEL2REG           0x32
#define     TESTPINENREG          0x33
#define     TESTPINVALUEREG       0x34
#define     TESTBUSREG            0x35
#define     AUTOTESTREG           0x36
#define     VERSIONREG            0x37
#define     ANALOGTESTREG         0x38
#define     TESTDAC1REG           0x39
#define     TESTDAC2REG           0x3A
#define     TESTADCREG            0x3B
#define     RESERVED31            0x3C
#define     RESERVED32            0x3D
#define     RESERVED33            0x3E
#define     RESERVED34            0x3F

//MF522 Command word
#define PCD_IDLE              0x00               //NO action; Cancel the current command
#define PCD_AUTHENT           0x0E               //Authentication Key
#define PCD_RECEIVE           0x08               //Receive Data
#define PCD_TRANSMIT          0x04               //Transmit data
#define PCD_TRANSCEIVE        0x0C               //Transmit and receive data,
#define PCD_RESETPHASE        0x0F               //Reset
#define PCD_CALCCRC           0x03               //CRC Calculate

// Mifare_One card command word
#define PICC_REQIDL          0x26               // find the antenna area does not enter hibernation
#define PICC_REQALL          0x52               // find all the cards antenna area
#define PICC_ANTICOLL        0x93               // anti-collision
#define PICC_SElECTTAG       0x93               // election card
#define PICC_AUTHENT1A       0x60               // authentication key A
#define PICC_AUTHENT1B       0x61               // authentication key B
#define PICC_READ            0x30               // Read Block
#define PICC_WRITE           0xA0               // write block
#define PICC_DECREMENT       0xC0               // debit
#define PICC_INCREMENT       0xC1               // recharge
#define PICC_RESTORE         0xC2               // transfer block data to the buffer
#define PICC_TRANSFER        0xB0               // save the data in the buffer
#define PICC_HALT            0x50               // Sleep

//MFRC-522 Cdigos de erros ocorridos durante a comunicao
#define MI_OK                 0    //tudo  'ok' (sem erro)
#define MI_NOTAGERR           1    //no existe tag no campo do leitor
#define MI_ERR                2    //erro na comunicao aconteceu.
#define MAX_LEN              16    //tamanho mximo 

//defines gerais
#define MODO_PROG  1 //valor para modo de programao
#define MODO_NORMAL 0 //valor para modo normal
#define TAGMESTREGRAVADA 170 //um nmero qualquer para indicar a condio
#define POSICAO_TAGMESTRE    2 //localizado tag mestre no endereo 2 da eeprom
#define TAGDESCONHECIDA  0x7e //tag ainda no foi gravada na eeprom
#define ESCREVER 0        //escrever dados
#define LER      1        //ler dados
#define DELETAR_EEPROM  0  //indica operao de deletar todas as tags
#define DELETAR_TAG_MESTRE 1 //indica operao de deletar tag mestre
#define DESLOCAMENTO  2  //indica deslocamento na posio inicial da eeprom

//declaraes de prottipos
void delay_piscal();//delay 200 ms
void delay_1seg();//delay de 1 segundo
void init_spi();//inicia a comunicao spi
void rst_mfrc522();//reseta MFRC-522
void init_mfrc522();//inicia MFRC-522
void testa_botao_del(unsigned int8 modo);//testa boto de deletar
void apagar_EEprom();//apaga toda a eeprom do PIC
void clock();//pulso de clock
void env_comando_spi(unsigned int8 end, unsigned int8 oper);//envia comandos
void env_dado_spi(byte dado);//envia dados SPI
unsigned int8 le_dado_spi();//recebe dado SPI
void setabit_mfrc522(unsigned int8 end, unsigned int8 mascara);//seta bit
void resetabit_mfrc522(unsigned int8 end, unsigned int8 mascara);//reseta bit
void liga_antena_mfrc522();//liga antena
void desliga_antena_mfrc522();//desliga antena
int8 Envia_Comando_para_Tag(byte cmd, byte *data, int8 dlen, byte *result, int8 *rlen);//envia comando para a tag
unsigned int8 procura_tag(unsigned int8 modo,unsigned int8 *data);//procura tag
void get_UID();//obtem o UID da tag no campo de ao do leitor
int anti_colisao(unsigned int8 *serial);//rotina de anticoliso para leitura da tag
unsigned int8 localiza_posicao_ID_gravado();//localiza tag gravada na eeprom

 //variveis globais
 int8 b,find,c;
 int1 f_mp=0;//flag modo de programao

unsigned int8 dados_tag[16];//array de envio/recepo de dados

/*******************************************************************************
                 Rotina de delay
*******************************************************************************/

void delay_piscal(){ delay_ms(200);}//rotina de delay de 200 ms
void delay_1seg(){ delay_ms(1000);}// rotina de delay de 1 segundo


/*******************************************************************************
           Rotinas Gerais de comunicao SPI
*******************************************************************************/
void init_spi(){//inicializao
  mosi=0;
  sck=0;
  sda=0;
  rst=0;//reseta
  delay_us(100);
  rst=1;//modo normal
  delay_ms(1);
}
/*******************************************************************************
           Rotina de gerao do pulso de clock na comunicao SPI
*******************************************************************************/
void clock(){
   sck=1;
   delay_us(1);//2   10=~100khz
   sck=0;
}
/*******************************************************************************
           Rotina de Envio de um Comando na Comunicao SPI
*******************************************************************************/
void env_comando_spi(unsigned int8 end, unsigned int8 oper){
 unsigned int8 reg,a;
   sda=0;
   reg=end<<1;
   if(oper)reg=reg|0x80;//bit7 tem que ser '1' para ler
   else reg=reg&0b01111110;//bit7 tem que ser '0' para escrever

   for(a=0;a<8;a++){
     if(reg&0b10000000)mosi=1; else mosi=0;
     reg=reg<<1;
     clock();
   }

}
/*******************************************************************************
           Rotina de Envio de 1 Byte na Comunicao SPI
*******************************************************************************/

void env_dado_spi(byte dado){
   unsigned int8 reg,a;
    reg=dado;
    for(a=0;a<8;a++){
     if(reg&0b10000000)mosi=1; else mosi=0;
     reg=reg<<1;
     clock();
   }

}

/******************************************************************************
           Rotina de Leitura de um dado na comunicao SPI
******************************************************************************/
unsigned int8 le_dado_spi(){
   unsigned int8 rc,a;
   for(a=0;a<8;a++){//le 8 bits
     rc=rc<<1;//desloca 1 bit para esquerda
     if(miso)rc++;
     clock();
   }
   return rc;//retorna o valor lido
}
/******************************************************************************
           Rotinas de Comunicao com MFRC522
******************************************************************************/
//liga a antena
void liga_antena_mfrc522(){
  setabit_mfrc522(0x14,0x03);
}

//desliga a antena
void desliga_antena_mfrc522(){
  resetabit_mfrc522(0x14,0x03);//TxControlReg
}

//reset por software
void rst_mfrc522(){
  env_comando_spi(COMMANDREG, ESCREVER);//escrever CommandReg
  env_dado_spi(0x0f);//resetar
  sda=1;
}

//inicializao do MFRC522
void init_mfrc522(){
  rst_mfrc522();//reseta mfrc522

  env_comando_spi(TMODEREG, ESCREVER);//escrever TModeReg
  env_dado_spi(0x8D);//tauto=1; f(timer)=6.78Mhz/tpresc
  sda=1;

  env_comando_spi(TPRESCALERREG, ESCREVER);//escrever TModeReg
  env_dado_spi(0x3E);//tmode[3...1]+TPrescallerReg
  sda=1;

  env_comando_spi(TRELOADREGL, ESCREVER);//
  env_dado_spi(30);//
  sda=1;

  env_comando_spi(TRELOADREGH, ESCREVER);//
  env_dado_spi(0);//
  sda=1;

  env_comando_spi(TXAUTOREG, ESCREVER);//100%ASK
  env_dado_spi(0x40);//
  sda=1;

  env_comando_spi(MODEREG, ESCREVER);//CRC valor inicial=0x6363
  env_dado_spi(0x3D);//
  sda=1;

  liga_antena_mfrc522();//liga antena
}

/*******************************************************************************
    Procura por uma tags passando pelo campo de leitura do MFRC522
    (rotina testada e funcionando!
*******************************************************************************/
unsigned int8 procura_tag(unsigned int8 modo,unsigned int8 *data){
  unsigned int8 status, len;

  env_comando_spi(BITFRAMINGREG, ESCREVER);//escrever BitFramingReg
  env_dado_spi(0x07);//// TxLastBists = BitFramingReg[2..0]
  sda=1;
  data[0] = modo;
  status = Envia_Comando_para_Tag(PCD_TRANSCEIVE, data, 1, data, &len);//

  if ((status != MI_OK) || (len != 0x10)) {
    status = MI_ERR;
  }
  return status;
}


/*******************************************************************************
    Rotina de envio de um comando para a tag    
*******************************************************************************/

int8 Envia_Comando_para_Tag(int8 cmd, int8 *data, int8 dlen, int8 *result, int8 *rlen) {
  int8 status = MI_ERR;
  int8 irqEn = 0x00;
  int8 waitIRq = 0x00;
  int8 lastBits, n;
  int8 i,a;

  switch (cmd) {
  case PCD_AUTHENT:
    irqEn = 0x12;
    waitIRq = 0x10;
    break;
  case PCD_TRANSCEIVE:
    irqEn = 0x77;
    waitIRq = 0x30;
    break;
  default:
    break;
  }

  env_comando_spi(COMMIENREG, ESCREVER); // requisio de interrupo
  env_dado_spi(irqEn|0x80);
  sda=1;

  resetabit_mfrc522(COMMIRQREG, 0x80);  //apaga bits de interrupes

  setabit_mfrc522(FIFOLEVELREG, 0x80);  // FlushBuffer=1, FIFO initialization.

  env_comando_spi(COMMANDREG, ESCREVER);// Nenhuma ao, cancela comandos.
  env_dado_spi(PCD_IDLE);//
  sda=1;

  //escreve para a FIFO
  for (i=0; i < dlen; i++) {
      env_comando_spi(FIFODATAREG, ESCREVER);
      env_dado_spi(data[i]);//
      sda=1;
  }

  //Envia um comando
  env_comando_spi(COMMANDREG, ESCREVER);//
  env_dado_spi(cmd);//
  sda=1;

  if (cmd == PCD_TRANSCEIVE) {
    setabit_mfrc522(BITFRAMINGREG, 0x80); // StartSend=1, inicia transmisso
  }

  // Aguarda o comando ser executado e depois podemos ler a FIFO.
  i = 250; // Tempo max. de espera 25ms.
  do {
       delay_us(100);//dealy de 100  useg
       env_comando_spi(COMMIRQREG,LER);//
       n=le_dado_spi();
       sda=1;//finaliza comunicao
       i--;
  } while ((i!=0) && !(n&0x01) && !(n&waitIRq));

   resetabit_mfrc522(BITFRAMINGREG, 0x80);// StartSend=0, termina comunicao

  if (i != 0) { // Confere se no houve time out.
    env_comando_spi(ERRORREG,LER);////BufferOvfl Collerr CRCErr ProtecolErr
    a=le_dado_spi();
     sda=1;//finaliza comunicao
     a=a & 0x1B; //mascara de bits a serem considerados

    if(!a){
      status = MI_OK;
      if (n & irqEn & 0x01) {
        status = MI_NOTAGERR;//no existe tag
      }

      if (cmd == PCD_TRANSCEIVE) {
        env_comando_spi(FIFOLEVELREG,LER);//
        n=le_dado_spi();
        sda=1;//finaliza comunicao

        env_comando_spi(CONTROLREG,LER);//
        lastbits=le_dado_spi();
        sda=1;//finaliza comunicao

        lastbits=lastbits & 0x07;
        if (lastBits) { *rlen = (n-1) * 8 + lastBits;}
        else { *rlen = n*8;}

        if (n == 0) { n = 1;}

        if (n > MAX_LEN) { n = MAX_LEN; }

        // Obter o streamming da FIFO.

        for (i=0; i<n; i++) {

          env_comando_spi(FIFODATAREG,LER);
          result[i] =le_dado_spi();
          sda=1;//finaliza comunicao
        }
        result[i]=0;
      }
    } else { status = MI_ERR; }
  }
  return status;
}

/*******************************************************************************
          Setar um bit usando um valor de mascaramento
*******************************************************************************/
void setabit_mfrc522(unsigned int8 end, unsigned int8 mascara){
  unsigned int8 lido;
  env_comando_spi(end,LER); //le registrador
  lido=le_dado_spi();//recebe byte
  sda=1;
  lido=lido|mascara;//seta bit usando 'or'
  env_comando_spi(end,ESCREVER); //escreve registrador
  env_dado_spi(lido);//escreve registrador com bit setado
  sda=1;
}
/*******************************************************************************
          Resetar um bit usando um valor de mascaramento
*******************************************************************************/
void resetabit_mfrc522(unsigned int8 end, unsigned int8 mascara){
  unsigned int8 lido;
  env_comando_spi(end,LER); //le registrador
  lido=le_dado_spi();//recebe byte
  sda=1;
  lido=lido&(~mascara);//reseta bit usando 'and'
  env_comando_spi(end,ESCREVER); //escreve registrador
  env_dado_spi(lido);//escreve registrador com bit resetado
  sda=1;
}

/*******************************************************************************
          Obter UID da tag no campo do leitor MFRC-522
*******************************************************************************/
int8 get_UID() {
   int8 a;
   a = anti_colisao(dados_tag);
   return a;
}
/*******************************************************************************
    Rotina para ler os 4 bytes da nica Identificao  (UID) da tag 
*******************************************************************************/

unsigned int8 anti_colisao(byte *mBuf) {
  unsigned int8 status, i, len;
  unsigned int8 checksum = 0x00;
  #define str_len   2

  env_comando_spi(BITFRAMINGREG,ESCREVER); //escreve registrador
  env_dado_spi(0x00);//
  sda=1;

  mBuf[0] = PICC_ANTICOLL;//comando anticoliso
  mBuf[1] = 0x20;

  status = Envia_Comando_para_Tag(PCD_TRANSCEIVE, mBuf, str_len, mBuf, &len);

  if (status == MI_OK) {

    for (i = 0; i < 4; i++) {//checksum= 0x00^mBuf[0]^mBuf[1]^mBuf[2]^mBuf[3]
      checksum ^= mBuf[i];// o checksum  Xor de todos os bytes
    }

    if (checksum != mBuf[4]) {//o checksum tem que ser igual ao 5 byte da tag
      status = MI_ERR;//se for diferente gera erro
    }
  }

  return status;
}

/******************************************************************************
       Rotinas de Busca do UID de uma tag gravada na EEprom do PIC
******************************************************************************/
 unsigned int8 localiza_posicao_ID_gravado(){
   unsigned int8 i,y;


   for(i=DESLOCAMENTO;i<126;i=i+4){
       for(y=0;y<4;y++){
         if(dados_tag[y]!=read_eeprom(y+i))break;
       }
       if(y==4)return i;//pega o endereo inicial do UID encontrado

   }
   return i; //se i=126 (0x7e) ento no encontrou
}

/******************************************************************************
        Rotinas de Gravao do UID da tag mestre na EEprom do PIC
******************************************************************************/
void gravartagmestre(){
 int8 i;
     for(i=0;i<4;i++){
     write_eeprom(i+DESLOCAMENTO,dados_tag[i]);
     }
     write_eeprom(1,TAGMESTREGRAVADA);
     f_mp=0; //desliga modo de programao
}

/******************************************************************************
       Rotinas de Gravao do UID das outras tags na EEprom do PIC
******************************************************************************/
void gravar_outras_tags(){
  unsigned int8 a,b;

  if(f_mp==MODO_PROG ){

   for(a=DESLOCAMENTO+4;a<126;a=a+4){//localiza posio vazia e grava UID da tag
       for(b=0;b<4;b++){
        if(read_eeprom(a+b)!=0xFF)break; //sai do lao secundrio
       }
       if(b==4){//grava UID a partir da posio do registrador 'a'
        for(b=0;b<4;b++){
          write_eeprom(a+b,dados_tag[b]);//grava UID na posio vazia
        }
        led_vm=LED_DESLIGADO;
        led_vd=LED_LIGADO;
        delay_1seg();
        led_vd=LED_DESLIGADO;
        break;//sai do lao primrio
       }
   }
  }
 else{//indica tag rejeitada
    led_vd=LED_DESLIGADO;
    led_vm=LED_LIGADO;
    delay_1seg();
    led_vm=LED_DESLIGADO;
  }
}
/*******************************************************************************
                 Rotina de Apagamento de EEprom do PIC
*******************************************************************************/
void apagar_EEprom(){
  unsigned int8 a;
  for( a=0;a<128;a++){
    write_eeprom(a,0XFF);//zera eeprom do pic
  }
}

//==============================================================================
//         Rotina de teste de boto 'Deletar'
//==============================================================================
void testa_botao_del(unsigned int8 modo){
 unsigned int8 t=0;

 while(!sw_del){
    led_vd=LED_DESLIGADO;
    led_vm=LED_LIGADO;
    delay_ms(100);
    t++;
    if(t>=100){

      if(modo==DELETAR_EEPROM){
         apagar_EEprom();
         t=3; //pisca led vermelho 3 vezes para indicar apagamento da EEprom
         while(t){
           delay_piscal();
           led_vm=LED_LIGADO;
           delay_piscal();
           led_vm=LED_DESLIGADO;
           t--;
         }

      }
      else{
         write_eeprom(1,0XFF);
         for(t=0;t<4;t++){
           write_eeprom(t+DESLOCAMENTO,0XFF);

         }
         t=5; //pisca led vermelho 5 vezes para indicar apagamento da EEprom
         while(t){
           delay_piscal();
           led_vm=LED_LIGADO;
           delay_piscal();
           led_vm=LED_DESLIGADO;
           t--;
         }

      }
      f_mp=MODO_PROG; //entra em modo de programao automaticamente
    }//if(t>=100)

  }//while(!sw_del)

  while(!sw_del); //aguarda soltar boto
  led_vm=LED_DESLIGADO;
}

//==============================================================================
//         ROTINA MAIN
//==============================================================================
void main() {
  unsigned int8 resp;
   setup_comparator(NC_NC_NC_NC);
   trisb=0b11101000;//sentido fixo das portas
   trisa=0b11110000;//idem
   porta=0b11111110;//desliga leds anodo comum e rele
   portb=1;
   init_spi();//inicializa comunicao spi
   init_mfrc522();
   if(read_eeprom(1)!=TAGMESTREGRAVADA) f_mp=MODO_PROG; else f_mp=MODO_NORMAL;
   testa_botao_del(DELETAR_EEPROM);//verifica se deve deletar toda a EEprom do PIC

//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================
   for(;;){
     testa_botao_del(DELETAR_TAG_MESTRE);//verifica se deve deletar tag mestre

     if(f_mp) {led_az=!led_az; }//comea a piscar led azul
     else {led_az=LED_LIGADO;}//modo normal led azul aceso
     resp=procura_tag(PICC_REQIDL,dados_tag);
     if(resp==MI_OK){
        b=get_UID();//obtem a identificao da tag (carto/chaveiro)

        if(b==MI_OK){
           if(read_eeprom(1) !=TAGMESTREGRAVADA){
           gravartagmestre();
           }
           else{
             find=localiza_posicao_ID_gravado();

             if(find==POSICAO_TAGMESTRE){//tag mestre encontrada
              f_mp=!f_mp;//alterna entre modo de programao e normal
              delay_ms(300);
              
             }
             else {//no  tag mestre
                    if(find!=TAGDESCONHECIDA){//encontrou tag na eeprom

                        if(f_mp==MODO_NORMAL){//liberar fechadura
                             led_vm=LED_DESLIGADO;
                             led_vd=LED_LIGADO;
                             rele=RELE_LIGADO;  // aciona fechadura
                             delay_1seg();      // por um segundo
                             led_vd=LED_DESLIGADO;
                             rele=RELE_DESLIGADO;
                        }
                        else{//apaga tag
                           for(c=0;c<4;c++){
                             write_eeprom(find+c,0xFF);
                           }
                           led_vm=LED_LIGADO;//pisca led vermelho para indicar
                           delay_1seg();
                           led_vm=LED_DESLIGADO;
                        }


                     }
                     else{//tag desconhecida: grava se modo de programao
                       gravar_outras_tags();//grava tags na eeprom

                     }
             }//else
           }//else
        }//if(b==MI_OK)
      }//if(resp==MI_OK)
     delay_piscal();//200 ms (determina a velocidade da piscada do led azul)
   }//for(;;)
}//main()
